# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import subprocess
import toml
# import sys
# sys.path.insert(0, os.path.abspath('.'))



# -- Project information -----------------------------------------------------
# obtain project information from pyproject.toml
# Path to your pyproject.toml file
pyproject_path = os.path.join(os.path.dirname(__file__), '..', 'pyproject.toml')

# Load the pyproject.toml file
with open(pyproject_path, 'r') as f:
    pyproject_data = toml.load(f)

# Extract the relevant fields from the pyproject.toml
project = pyproject_data['project']['name']
authors = pyproject_data['project']['authors']
author = ', '.join([author['name'] for author in authors])  # Comma-separated list of author names
release = pyproject_data['project']['version']
copyright = f"2024, {author}"  # Use author and release for copyright

# -- General configuration ---------------------------------------------------

# Settings to determine if we are building on readthedocs
read_the_docs_build = os.environ.get('READTHEDOCS', None) == 'True'
if read_the_docs_build:
    subprocess.call('pwd', shell=True)
    subprocess.call('ls', shell=True)
    subprocess.call('cmake -B ../build -S .. -DBUILD_DOCS=ON', shell=True)
    subprocess.call('cmake --build ../build  --target Doxygen', shell=True)
    xml_dir = os.path.abspath("../build/docs/xml")

    

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',  # For Google/NumPy-style docstrings
    'sphinx.ext.viewcode',  # Links to source code
    'breathe',              # For C++ API
]

# Breathe Configuration
breathe_default_project = "polytopewalk"
if read_the_docs_build:
    breathe_projects = {"polytopewalk": xml_dir}


# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

autoclass_content = 'both'


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


import unittest
from polytopewalk import FacialReduction
import numpy as np 
import pandas as pd
from scipy.sparse import lil_matrix
from polytopewalk.sparse import *
from polytopewalk.dense import *

simplex_A = lil_matrix((1, 3))
simplex_A[(0, 0)] = 1
simplex_A[(0, 1)] = 1
simplex_A[(0, 2)] = 1
simplex_b = np.array([1])

hc_A = lil_matrix((4,6))
hc_A[(0, 0)] = 1
hc_A[(0, 2)] = 1
hc_A[(1, 1)] = 1
hc_A[(1, 3)] = 1
hc_A[(2, 0)] = -1
hc_A[(2, 4)] = 1
hc_A[(3, 1)] = -1
hc_A[(3, 5)] = 1

hc_b = np.array([1,1,1,1])

birk_A = lil_matrix((3, 4))
birk_A[(0, 0)] = 1
birk_A[(0, 1)] = 1
birk_A[(1, 2)] = 1
birk_A[(1, 3)] = 1
birk_A[(2, 0)] = 1
birk_A[(2, 2)] = 1

birk_b = np.array([1,1,1])

class TestWeights(unittest.TestCase):
    def test_weights(self):
        sparse_vaidya = SparseVaidyaWalk(r = 0.9, thin = 1)
        sparse_john = SparseJohnWalk(r = 0.9, thin = 1)
        sparse_dikinls = SparseDikinLSWalk(r = 0.9, thin = 1)

        simplex_start = np.array([0.33, 0.34, 0.33])
        w = sparse_dikinls.generateWeight(simplex_start, simplex_A, 3)
        self.assertAlmostEqual(w.sum(), 2, places = 1)
        w = sparse_john.generateWeight(simplex_start, simplex_A, 3)
        self.assertAlmostEqual(w.sum(), 3, places = 2)
        w = sparse_vaidya.generateWeight(simplex_start, simplex_A, 3)
        self.assertAlmostEqual(w.sum(), 4, places = 2)

        hc_start = np.array([0, 0, 1, 1, 1, 1])
        w = sparse_dikinls.generateWeight(hc_start, hc_A, 4)
        self.assertAlmostEqual(w.sum(), 2, places = 1)
        w = sparse_john.generateWeight(hc_start, hc_A, 4)
        self.assertAlmostEqual(w.sum(), 3, places = 2)
        w = sparse_vaidya.generateWeight(hc_start, hc_A, 4)
        self.assertAlmostEqual(w.sum(), 4, places = 2)

        birk_start = np.array([0.5, 0.5, 0.5, 0.5])
        w = sparse_dikinls.generateWeight(birk_start, birk_A, 4)
        self.assertAlmostEqual(w.sum(), 1, places = 1)
        w = sparse_john.generateWeight(birk_start, birk_A, 4)
        self.assertAlmostEqual(w.sum(), 1.5, places = 2)
        w = sparse_vaidya.generateWeight(birk_start, birk_A, 4)
        self.assertAlmostEqual(w.sum(), 2, places = 2)


if __name__ == '__main__':
    unittest.main()


import unittest
from polytopewalk import FacialReduction
import numpy as np 
import pandas as pd
from scipy.sparse import lil_matrix
from polytopewalk.sparse import *
from polytopewalk.dense import *

simplex_A = lil_matrix((1, 3))
simplex_A[(0, 0)] = 1
simplex_A[(0, 1)] = 1
simplex_A[(0, 2)] = 1
simplex_b = np.array([1])

hc_A = lil_matrix((4,6))
hc_A[(0, 0)] = 1
hc_A[(0, 2)] = 1
hc_A[(1, 1)] = 1
hc_A[(1, 3)] = 1
hc_A[(2, 0)] = -1
hc_A[(2, 4)] = 1
hc_A[(3, 1)] = -1
hc_A[(3, 5)] = 1

hc_b = np.array([1,1,1,1])

birk_A = lil_matrix((3, 4))
birk_A[(0, 0)] = 1
birk_A[(0, 1)] = 1
birk_A[(1, 2)] = 1
birk_A[(1, 3)] = 1
birk_A[(2, 0)] = 1
birk_A[(2, 2)] = 1

birk_b = np.array([1,1,1])

class TestFacialReduction(unittest.TestCase):

    def test_fr(self):
        fr = FacialReduction()
        simplex_dense = fr.reduce(simplex_A, simplex_b, 3, False)
        hc_dense = fr.reduce(hc_A, hc_b, 4, False)
        birk_dense = fr.reduce(birk_A, birk_b, 4, False)

        self.assertTrue(simplex_dense.dense_A.shape == (3,2))
        self.assertTrue(len(simplex_dense.dense_b) == 3)

        self.assertTrue(hc_dense.dense_A.shape == (4,2))
        self.assertTrue(len(hc_dense.dense_b) == 4)

        self.assertTrue(birk_dense.dense_A.shape == (4,1))
        self.assertTrue(len(birk_dense.dense_b) == 4)

        A = np.array([[1, 1, 0], [-1, -1, 0], [0, 1, 0], [0, -1, 0], [0, 0, 1], [0, 0, -1]])
        A = np.hstack((A, np.eye(6)))
        b = np.array([1, -1, 1, 1, 1, 1])

        fr_res = fr.reduce(A, b, 6, False)
        self.assertTrue(fr_res.sparse_A.shape == (5, 7))
        self.assertTrue(fr_res.dense_A.shape == (4, 2))


        A = np.array([[1, 0, 0], [-1, 0, 0], [0, 1, 0], [0, -1, 0], [0, 0, 1], [0, 0, -1]])
        A = np.hstack((A, np.eye(6)))
        b = np.array([1, 1, 0, 0, 0, 0])

        fr_res = fr.reduce(A, b, 6, False)
        self.assertTrue(fr_res.sparse_A.shape == (4, 5))
        self.assertTrue(fr_res.dense_A.shape == (2, 1))

        A = np.array([[1, 0], [-1, 0], [0, 1], [0, -1]])
        A = np.hstack((A, np.eye(4)))
        b = np.array([1, 0, 1, 0])

        fr_res = fr.reduce(A, b, 4, False)
        self.assertTrue(fr_res.sparse_A.shape == (4, 6))
        self.assertTrue(fr_res.dense_A.shape == (4, 2))


    
if __name__ == '__main__':
    unittest.main()

